import pathlib
import git
import os
from os.path import join
from datetime import datetime

from PyQt5.QtCore import QObject


class Model(QObject):
    def __init__(self):
        super().__init__()
        self._record_path = ""
        self._record_interval = 3

    @property
    def record_path(self):
        """Return the value of record_path."""
        return self._record_path

    @record_path.setter
    def record_path(self, record_path):
        """Set the value of record_path."""
        self._record_path = record_path

    @property
    def record_interval(self):
        return self._record_interval

    @record_interval.setter
    def record_interval(self, interval):
        self._record_interval = interval

    def get_dotlup_path(self):
        """Return the value of watchers."""
        return join(self._record_path, ".lup")

    def create_dotlup_dir(self):
        """Create watchers directory."""
        pathlib.Path(join(self._record_path, ".lup")).mkdir(parents=True, exist_ok=True)

    def write_auth_info(self, auth_info):
        """Write auth information to file."""
        auth_file_path = join(self.get_dotlup_path(), "auth_info")
        auth_file = open(auth_file_path, "w+")
        auth_file.write(auth_info)
        auth_file.close()

    def write_all_windows(self, all_windows):
        """Write all windows title to file."""
        all_windows_path = join(self.get_dotlup_path(), "all_windows")
        all_windows_file = open(all_windows_path, "w+")
        all_windows_file.write(all_windows)
        all_windows_file.close()

    def write_focused_window(self, active_window):
        """Write focused window title to file."""
        focused_window_path = join(self.get_dotlup_path(), "focused_window")
        focused_window_file = open(focused_window_path, "w+")
        focused_window_file.write(active_window)
        focused_window_file.close()

    def is_repo(self):
        if os.path.isdir(join(self._record_path, ".git")):
            return True

    def initialize_student_repo(self):
        student_repo = git.Repo.init(self._record_path)
        # real name and email useless cause lup already record login
        # and machine name
        student_repo.config_writer().set_value("user", "name", "student").release()
        student_repo.config_writer().set_value("user", "email", "s@student").release()
        return student_repo

    def get_student_repo(self):
        if self.is_repo():
            student_repo = git.Repo(self._record_path)
        else:
            student_repo = self.initialize_student_repo()
        return student_repo

    def create_record(self):
        student_repo = self.get_student_repo()
        # commit only when changed file present
        if "nothing to commit" not in str(student_repo.git.status()):
            student_repo.git.add(".")
            now = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            student_repo.git.commit(m=now)



import sys
from PyQt5.QtWidgets import QApplication

from Lupr.model.model import Model
from Lupr.controllers.controller import Controller
from Lupr.views.main_view import MainView


class Lupr(QApplication):
    def __init__(self, sys_argv):
        super().__init__(sys_argv)
        self.model = Model()
        self.controller = Controller(self.model)
        self.main_view = MainView(self.model, self.controller)


def main():
    lupr = Lupr(sys.argv)
    lupr.setQuitOnLastWindowClosed(False)
    sys.exit(lupr.exec_())


if __name__ == "__main__":
    main()



import time
from PyQt5.QtCore import QThread


class RecordWorker(QThread):
    """Worker thread to record all activity.
    App need multi thread in order to hang. """

    def __init__(self, model, controller):
        QThread.__init__(self)
        self._model = model
        self._main_ctrl = controller

    def __del__(self):
        self.wait()

    def run(self):
        while True:
            auth_info = self._main_ctrl.get_auth_info()
            all_windows = self._main_ctrl.get_all_windows()
            focused_window = self._main_ctrl.get_focused_window()
            self._model.write_auth_info(auth_info)
            self._model.write_all_windows(all_windows)
            self._model.write_focused_window(focused_window)
            self._model.create_record()
            interval = self._model.record_interval
            time.sleep(interval - time.time() % interval)



from PyQt5.QtWidgets import QDialog
from Lupr.views.interval_prompt_ui import Ui_Dialog


class IntervalPrompt(QDialog, Ui_Dialog):
    def __init__(self):
        super().__init__()
        self.setupUi(self)

        self.cancel_btn.clicked.connect(self.close)
        self.apply_btn.clicked.connect(self.accept)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/media/azzamsya/rhd/azzamsa/self-management/references/masterpiece/software/code/projects/lup/lupr/lupr/views/interval_prompt.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(281, 82)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.interval_spin = QtWidgets.QSpinBox(Dialog)
        self.interval_spin.setMaximum(10000)
        self.interval_spin.setObjectName("interval_spin")
        self.verticalLayout.addWidget(self.interval_spin)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.apply_btn = QtWidgets.QPushButton(Dialog)
        self.apply_btn.setObjectName("apply_btn")
        self.horizontalLayout.addWidget(self.apply_btn)
        self.cancel_btn = QtWidgets.QPushButton(Dialog)
        self.cancel_btn.setObjectName("cancel_btn")
        self.horizontalLayout.addWidget(self.cancel_btn)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Set Interval"))
        self.apply_btn.setText(_translate("Dialog", "apply"))
        self.cancel_btn.setText(_translate("Dialog", "Cancel"))




import getpass
import socket
import re
from subprocess import Popen, PIPE
from PyQt5.QtCore import QObject


class Controller(QObject):
    def __init__(self, model):
        super().__init__()

        self._model = model

    def set_record_path(self, record_path):
        """Set record path then create dotlup dir."""
        self._model.record_path = record_path
        self._model.create_dotlup_dir()

    def change_interval(self, interval):
        """Change recording interval."""
        self._model.record_interval = interval

    def _handle_various_xprop(self, active_window):
        """Handle various xprop formats."""
        active_window_str = active_window
        if type(active_window) is bytes:
            active_window_str = active_window.decode()

        window_id = active_window_str.split()[-1]
        # handle XFCE xprop format
        if window_id == "0x0":
            window_id = active_window_str.split()[-2].strip(",")

        return window_id

    def get_focused_window_id(self):
        """Return focused windows id."""
        active_window_proc = Popen(
            ["xprop", "-root", "_NET_ACTIVE_WINDOW"], stdout=PIPE
        )
        active_window, err = active_window_proc.communicate()
        focused_window_id = self._handle_various_xprop(active_window)
        return focused_window_id

    def get_focused_window(self):
        """Get current focused window title.

        window_name_dirty value is  b\'WM_NAME(STRING) = "WINDOW TITLE"\n'.
        WINDOW TITLE will be parsed using regex and returned as string.
        new line at the end of file needed as it's git convention.
        """
        focused_window_id = self.get_focused_window_id()
        window_name_proc = Popen(
            ["xprop", "-id", focused_window_id, "WM_NAME"], stdout=PIPE
        )
        window_name_dirty, err = window_name_proc.communicate()
        focused_window = re.findall(r"\"(.+?)\"", window_name_dirty.decode())
        return "{}\n".format("".join(focused_window))

    def get_public_ip(self):
        """Return public ip address by connecting to google DNS."""
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(("8.8.8.8", 80))
            public_ip, _ = s.getsockname()
            s.close()
        except Exception:
            public_ip = "Not connected"
        return public_ip

    def get_all_windows(self):
        """Get all windows title.

        wmctrl result is 0x03800004  0 machine user - WINDOW TITLE.
        WINDOW TITLE will be taken and returned as string.
        """
        all_windows = ""
        all_windows_proc = Popen(["wmctrl", "-l"], stdout=PIPE)
        all_windows_dirty, err = all_windows_proc.communicate()
        for line in all_windows_dirty.splitlines():
            windows_name = line.split(None, 3)[-1].decode()
            all_windows += "{}\n".format(windows_name)
        return all_windows

    def get_auth_info(self):
        """Get auth information."""
        auth_info = ""
        username = getpass.getuser()
        machine = socket.gethostname()
        ip = self.get_public_ip()
        for data in [username, machine, ip]:
            auth_info += "{}\n".format(data)
        return auth_info



import sys

from PyQt5.QtWidgets import (
    QMenu,
    QSystemTrayIcon,
    QMainWindow,
    QFileDialog,
    QMessageBox,
)
from PyQt5.QtGui import QIcon

from Lupr.resources import images
from Lupr.worker.record_worker import RecordWorker
from Lupr.views.interval_prompt_view import IntervalPrompt


class MainView(QMainWindow):
    def __init__(self, model, controller):
        super().__init__()

        self._model = model
        self._controller = controller

        # UI
        icon = QIcon(":img/lup.svg")
        menu = QMenu()
        self.record_action = menu.addAction("Start Recording")
        self.set_interval_action = menu.addAction("Set Interval")
        self.quit_action = menu.addAction("Stop and Quit")
        self.tray = QSystemTrayIcon(self)
        self.tray.setIcon(icon)
        self.tray.setContextMenu(menu)
        self.tray.show()
        self.tray.setToolTip("Lup")
        self.tray.showMessage("Lup", "Welcome to Lup", self.tray.Information, 1500)

        self.record_action.triggered.connect(self.record)
        self.set_interval_action.triggered.connect(self.set_interval)
        self.quit_action.triggered.connect(self.quit_app)

        self.recorderThread = RecordWorker(self._model, self._controller)

        # default
        self.set_interval_action.setEnabled(False)

    def choosedir_dialog(self, caption):
        """Prompts dialog to choose record directory."""
        options = QFileDialog.ShowDirsOnly | QFileDialog.DontResolveSymlinks
        return QFileDialog.getExistingDirectory(self, caption=caption, options=options)

    def record(self):
        """Start recording with worker(recorder) thread."""
        path = self.choosedir_dialog("Select Directory...")
        if not path:
            return None

        self._controller.set_record_path(path)
        self.recorderThread.start()

        self.set_interval_action.setEnabled(True)
        self.record_action.setEnabled(False)
        self.tray.showMessage("Lup", "Lup is recording", self.tray.Information, 1500)

    def set_interval(self):
        "Change record interval."
        interval_prompt = IntervalPrompt()
        accepted = interval_prompt.exec_()

        if accepted:
            interval = interval_prompt.interval_spin.value()
            if interval:
                self._controller.change_interval(interval)
                self.tray.showMessage(
                    "Lup",
                    "Interval chaned to {}".format(interval),
                    self.tray.Information,
                    1500,
                )
            else:
                title = "Interval Warning"
                msg = "Interval must higher than 0"
                QMessageBox.warning(None, title, msg)
                self.set_interval()

    def quit_app(self):
        """Quit the app."""
        self.recorderThread.terminate()
        self.tray.showMessage("Lup", "See you")
        sys.exit()
